-----------------------------------------
--                RageBar                  --
--     An additional rage bar     --
----------------------------------------

RB = LibStub("AceAddon-3.0"):NewAddon("RageBar", "AceEvent-3.0", "AceConsole-3.0")
local media = LibStub("LibSharedMedia-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("RageBar")
local textures = media:List("statusbar")
local fonts = media:List("font")
local RageCap
local AceConfig
local RBBar
local RageText
local LossText
local RBFrame
local Temp

-- default options
local defaults = {
	global = {
		width = 250,
	 	height = 20,
		barori = "HORIZONTAL",
	 	fontsize = 20,
		ragetext = "LEFT",
		losstext = "RIGHT",
	 	lock = false,
	 	x = 0,
	 	y = 0,
		hideooc = false,
		hideoor = false,
		barmax = false,
		vertical = false,
		colorbg = {
			["r"] = 0,
			["g"] = 0,
			["b"] = 0,
			["a"] = 1,		 
		},
		font = 1,
		tex = 1,
		talentmaxrank = 3,
		talentrank = 0,
		talenticon = "Interface\\Icons\\Spell_Nature_EnchantArmor",
		talentname = "Tactical Mastery",
	},
}

function RB:OnInitialize()
	local _, Class = UnitClass("player");
	if Class ~= "WARRIOR" then RB:Print(L["has not been loaded, you are no warrior!"]) return end
	RB.db = LibStub('AceDB-3.0'):New('RageBardb')
	RB.db:RegisterDefaults(defaults)
	RB:Options()
	RageCap = (RB.db.global.talentrank*5)+10
	RB:Print(L["RageBar v1.0 loaded."])
end

-- Frame dragging
local function dragstart()
	if not RB.db.global.lock then
		RBFrame:StartMoving()
	end
end

local function dragstop()
	RBFrame:StopMovingOrSizing()
	RB.db.global.x = RBFrame:GetLeft()
	RB.db.global.y = RBFrame:GetBottom()
end
	
function RB:OnEnable()
	local _, Class = UnitClass("player");
	if Class ~= "WARRIOR" then return end 
	self:RegisterEvent("PLAYER_REGEN_DISABLED")
	self:RegisterEvent("PLAYER_REGEN_ENABLED")
	self:RegisterEvent("UNIT_RAGE")
	self:RegisterEvent("CHARACTER_POINTS_CHANGED")
	self:RegisterEvent("PLAYER_ENTERING_WORLD")

--Main Frame creation
	RBFrame = CreateFrame("Frame",nil,UIParent)
	RBFrame:SetFrameStrata("DIALOG")
	RBFrame:SetWidth(RB.db.global.width)
	RBFrame:SetHeight(RB.db.global.height)
	RBFrame:SetPoint("BOTTOMLEFT",RB.db.global.x,RB.db.global.y)
	RBFrame:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",tile = false})
	RBFrame:SetBackdropColor(RB.db.global.colorbg.r, RB.db.global.colorbg.g, RB.db.global.colorbg.b, RB.db.global.colorbg.a)
	RBFrame:SetMovable(true)
	RBFrame:RegisterForDrag('LeftButton')
	RBFrame:SetClampedToScreen(true)
	RBFrame:EnableMouse(not RB.db.global.lock)
	RBFrame:SetScript('OnDragStart', dragstart)
	RBFrame:SetScript('OnDragStop', dragstop)

--RageBar showing current rage
	RBBar = CreateFrame("StatusBar",nil,RBFrame)
	RBBar:SetStatusBarTexture(media:Fetch("statusbar",RB.db.global.tex))
	RBBar:SetOrientation(RB.db.global.barori)
	if not RB.db.global.barmax then RBBar:SetMinMaxValues(0, 100) else RBBar:SetMinMaxValues(0, RageCap) end 
	RBBar:SetValue(0)
	RBBar:SetAllPoints(RBFrame)
	
--Labels on the status bar
	RageText = RBBar:CreateFontString(nil, 'OVERLAY')
	RageText:SetPoint(RB.db.global.ragetext, RBFrame, 0, 0);
	RageText:SetFont(media:Fetch("font",RB.db.global.font),RB.db.global.fontsize)
	RageText:SetText(UnitMana('player'))

	LossText = RBBar:CreateFontString(nil, 'OVERLAY')
	LossText:SetPoint(RB.db.global.losstext, RBFrame, 0, 0);
	LossText:SetFont(media:Fetch("font",RB.db.global.font),RB.db.global.fontsize)
	LossText:SetText("|cff00ff000|r")

	if RB.db.global.hideooc then RBFrame:Hide() else RBFrame:Show() end
end

function RB:PLAYER_REGEN_DISABLED()
	RBFrame:Show()
end

function RB:PLAYER_REGEN_ENABLED()
	if UnitMana("player") == 0 and RB.db.global.hideoor then RBFrame:Hide() elseif RB.db.global.hideooc and not RB.db.global.hideoor then RBFrame:Hide() end
end

-- Get new values if the player spends talent points
function RB:CHARACTER_POINTS_CHANGED()
	RB.db.global.talentname, RB.db.global.talenticon, _, _, RB.db.global.talentrank, RB.db.global.talentmaxrank = GetTalentInfo(1, 6)
end

-- When zoning with a loading screen, rage is always reset to 0 but UNIT_RAGE is not triggered, so fix this with this event
function RB:PLAYER_ENTERING_WORLD()
	RB:UNIT_RAGE("player","player")
end

-- Changing rage values
function RB:UNIT_RAGE(arg1,arg2)
	if arg2 ~= "player" then return end
	local rage = UnitMana("player")
	local loss = rage-RageCap
	if loss < 1 then loss = "|cff00ff000|r" else loss = "|cffffff00"..(loss*-1).."|r" end
	RageText:SetText(tostring(rage))
	LossText:SetText(loss)
	RBBar:SetValue(rage)
	if rage >= RageCap then RBBar:SetStatusBarColor(1,0,0,1) else 	RBBar:SetStatusBarColor(0,1,0,1) end
	if rage > 0 and RB.db.global.hideoor then RBFrame:Show() elseif rage == 0 and RB.db.global.hideooc then RBFrame:Hide() end
end

-- /ragebar
function RB:OnCmd()
	RB.db.global.talentname, RB.db.global.talenticon, _, _, RB.db.global.talentrank, RB.db.global.talentmaxrank = GetTalentInfo(1, 6)
	InterfaceOptionsFrame_OpenToCategory("RageBar")
end

-- Helper function for shared media
local function GetLSMIndex(t, value)
	for k, v in pairs(media:List(t)) do
		if v == value then
			return k
		end
	end
	return nil
end

-- Options menu
function RB:Options()
local options = {
	type = "group",
	name = "RageBar",
	childGroups = "tab",
	args = {
	  	main = {
			type = "group",
			name = L["Common"],
			args = {
				header = {
					type = 'description',
					name = " "..RB.db.global.talentname..": "..RB.db.global.talentrank.."/"..RB.db.global.talentmaxrank.."\n "..L["Rage kept"]..": "..tostring((RB.db.global.talentrank*5)+10),
					imageWidth = 25,
					imageHeight = 25,
					image = RB.db.global.talenticon,
					order = 1,
				},
				about = {
					type = 'description',
					name = L["\n\nRageBar v1.0 by Navy EU Dalaran.\nAn additional rage bar, which shows how much rage would be lost by switching the stance.\n\nThe first time you load RageBar, you must reload UI to save Tactycal mastery rank\n\n"],
					order = 2,
				},
				lock = {
					type = 'toggle',
					name = L["Frame lock"],
					desc = L["Locks the frame / unlocks the frame."],
					set = function(info, value)
					RB.db.global.lock = value;
					RBFrame:EnableMouse(not RB.db.global.lock)
 					if not RB.db.global.lock then RB:Print(L["unlocked."]) else RB:Print(L["locked."]) end
					end,
					get = function(info) return RB.db.global.lock end,
					order = 3,
				},
				nothing = {
					type = 'description',
					name = "   ",
					order = 4,
				},
				reloadui = {
					type = 'execute',
					name = L["Reload UI"],
					desc = L["Save all preferences and reload user interface"],
					func = function()
						ReloadUI();
					end,
					order = 5,
				},
				reset = {
					type = 'execute',
					name = 'Reset',
					desc = L["Resets all preferences."],
					func = function()
						RB.db:ResetDB('global')
						RBFrame:SetBackdropColor(RB.db.global.colorbg.r, RB.db.global.colorbg.g, RB.db.global.colorbg.b, RB.db.global.colorbg.a);
						RageText:SetFont("Fonts\\ARIALN.TTF",RB.db.global.fontsize);
						LossText:SetFont("Fonts\\ARIALN.TTF",RB.db.global.fontsize);
						RBFrame:SetHeight(RB.db.global.height);
						RBFrame:SetWidth(RB.db.global.width);
						RageText:SetPoint(RB.db.global.ragetext, RBFrame, 0, 0);
						LossText:SetPoint(RB.db.global.losstext, RBFrame, 0, 0);
						RBBar:SetOrientation(RB.db.global.barori);
						RBFrame:SetPoint("BOTTOMLEFT",RB.db.global.x,RB.db.global.y);
						RBFrame:EnableMouse(not RB.db.global.lock);
						RBFrame:Show();
						RB:Print(L["Preferences reset."]);
					end,
					order = 6,
				},
			},
			order = 1,
		},
		frameopts = {
			type = "group",
			name = L["Bar options"],
			desc = L["Configure bar options"],
			args = {
				hideooc = {
					type = 'toggle',
					name = L["Show only in combat"],
					desc = L["Show only if in combat?"],
					set = function(info, value)
					RB.db.global.hideooc = value;
 					if not RB.db.global.hideooc then RB.db.global.hideoor = value RBFrame:Show() RB:Print(L["always visible."]) else RBFrame:Hide(); RB:Print(L["only visible in combat."]) end
					end,
					get = function(info) return RB.db.global.hideooc end,
					order = 1,
				},
				hideoor = {
					type = 'toggle',
					name = L["Show if you have rage"],
					desc = L["Show RageBar as long as you have rage after combat"],
					set = function(info, value)
					RB.db.global.hideoor = value;
 					if not RB.db.global.hideoor then RB:Print(L["only visible in combat."]) else RB.db.global.hideooc = value RBFrame:Hide() RB:Print(L["only visible in combat and as long as you have rage."]) end
					end,
					get = function(info) return RB.db.global.hideoor end,
					order = 2,
				},
  			    barmax = {
					type = 'toggle',
					name = L["Full bar "]..tostring((RB.db.global.talentrank*5)+10)..L[" rage"],
					desc = L["Full bar = rage kept when switching the stance or 100 rage?"],
					set = function(info, value)
					RB.db.global.barmax = value;
 					if not RB.db.global.barmax then RB:Print(L["Maximum bar value"].." = 100"..L[" rage"]) RBBar:SetMinMaxValues(0, 100) else RB:Print(L["Maximum bar value"].." = "..L["Rage kept"]) RBBar:SetMinMaxValues(0, (RB.db.global.talentrank*5)+10) end
					end,
					get = function(info) return RB.db.global.barmax end,
					order = 3,
				},
				vertical = {
					type = 'toggle',
					name = L["Vertical bar"],
					desc = L["Displays vertical bar"],
					set = function(info, value)
					RB.db.global.vertical = value;
					if not RB.db.global.vertical
						then
						RB:Print(L["horizontal"]);
						Temp = RB.db.global.width;
						RB.db.global.width = RB.db.global.height;
						RB.db.global.height = Temp;
						RBFrame:SetWidth(RB.db.global.width);
						RBFrame:SetHeight(RB.db.global.height);
						RB.db.global.ragetext = "LEFT";
						RB.db.global.losstext = "RIGHT";
						RageText:SetPoint(RB.db.global.ragetext, RBFrame, 0, 0);
						LossText:SetPoint(RB.db.global.losstext, RBFrame, 0, 0);
						RB.db.global.barori = "HORIZONTAL";
						RBBar:SetOrientation(RB.db.global.barori);
						else
						RB:Print(L["vertical"]);
						Temp = RB.db.global.width;
						RB.db.global.width = RB.db.global.height;
						RB.db.global.height = Temp;
						RBFrame:SetWidth(RB.db.global.width);
						RBFrame:SetHeight(RB.db.global.height);
						RB.db.global.ragetext = "BOTTOM";
						RB.db.global.losstext = "TOP";
						RageText:SetPoint(RB.db.global.ragetext, RBFrame, 0, 0);
						LossText:SetPoint(RB.db.global.losstext, RBFrame, 0, 0);
						RB.db.global.barori = "VERTICAL";
						RBBar:SetOrientation(RB.db.global.barori);
						end
					end,
					get = function(info) return RB.db.global.vertical end,
					order = 4,
				},
				width = {
					type = 'range',
					name = L["Bar width"],
					desc = L["Changes bar width."],
					min = 1,
					max = 500,
					step = 1,
					get = function() return RB.db.global.width end,
					set = function(info, v) RB.db.global.width = v; RBFrame:SetWidth(v);end, 
					order = 5,
				},
				height = {
					type = 'range',
					name = L["Bar height"],
					desc = L["Changes bar height."],
					min = 1,
					max = 500,
					step = 1,
					get = function() return RB.db.global.height end,
					set = function(info, v) RB.db.global.height = v; RBFrame:SetHeight(v); end,
					order = 6,
				},
				colorbg = {
					type = 'color',
					name = L["Background colour"],
					desc = L["Changes background colour and opacity."],
					hasAlpha = true,
					get = function(info) return RB.db.global.colorbg.r, RB.db.global.colorbg.g, RB.db.global.colorbg.b, RB.db.global.colorbg.a end,
					set = function(info, r, g, b, a)
						RBFrame:SetBackdropColor(r, g, b, a)
						RB.db.global.colorbg.r = r;
						RB.db.global.colorbg.g = g;
						RB.db.global.colorbg.b = b;
						RB.db.global.colorbg.a = a;
						end,
					order = 7,
				},
				bartex = {
					type = "select",
					name = L["Bar texture"],
					desc = L["Changes the bar texture used."],
					values = textures,
					get = function(info) return GetLSMIndex("statusbar", RB.db.global.tex) end,
					set = function(info, v)
						RB.db.global.tex = media:List("statusbar")[v]
  						RBBar:SetStatusBarTexture(media:Fetch("statusbar",RB.db.global.tex))
					end,
					order = 8,
				},
			},
			order = 2,
		},
		fontopts = {
			type = "group",
			name = L["Font options"],
			desc = L["Configure font options"],
			args = {
				font = {
					type = "select",
					name = L["Font"],
					desc = L["Changes the font used."],
					values = fonts,
					get = function(info) return GetLSMIndex("font", RB.db.global.font) end,
					set = function(info, v)
						RB.db.global.font = media:List("font")[v]
						RageText:SetFont(media:Fetch("font",RB.db.global.font),RB.db.global.fontsize)
						LossText:SetFont(media:Fetch("font",RB.db.global.font),RB.db.global.fontsize)												
						end,
					order = 1,
				},
				fontsize = {
					type = 'range',
					name = L["Font size"],
					desc = L["Changes the font size."],
					min = 1,
					max = 30,
					step = 1,
					get = function() return RB.db.global.fontsize end,
					set = function(info, v) RB.db.global.fontsize = v; RageText:SetFont(media:Fetch("font",RB.db.global.font),v); LossText:SetFont(media:Fetch("font",RB.db.global.font),v) end, 
					order = 2,
				},
			},
			order = 3,
		},
	},
}

-- Config Stuff
AceConfig = LibStub("AceConfigDialog-3.0")
LibStub("AceConfig-3.0"):RegisterOptionsTable("RageBar", options, nil)
AceConfig:AddToBlizOptions("RageBar", "RageBar")
RB:RegisterChatCommand('ragebar', 'OnCmd') 
end